# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinContentSupport library

    @copyright: 2008, 2009, 2010, 2011, 2013 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.wikiutil import escape
import re

__version__ = "0.1"

# Regular expressions.

heading_regexp_str = r"^(?P<level>=+)\s*(?P<heading>.*?)\s*(?P=level)$"
hrule_regexp_str = r"^----$"
include_regexp_str = r"^\s*<<Include.*?>>$"
section_regexp_str = "(" + heading_regexp_str + "|" + hrule_regexp_str + "|" + include_regexp_str + ")"
section_regexp = re.compile(section_regexp_str, re.UNICODE | re.MULTILINE)

# NOTE: This overlaps with EventAggregator.

category_membership_str = ur"^\s*(?:(Category\S+)(?:\s+(Category\S+))*)\s*$"
category_membership_regexp = re.compile(category_membership_str, re.MULTILINE | re.UNICODE)
category_declarations_regexp = re.compile("^----$\s*" + category_membership_str, re.MULTILINE | re.UNICODE)

def getSectionDetails(body, min_level=None, max_level=None):

    """
    Return section details from the given 'body' for headings with the given
    'min_level' to 'max_level' range. Specifying None or omitting 'max_level' or
    'min_level' removes the appropriate constraint on the range.

    A list of tuples is returned for each section divider. For headings, the
    heading text and level number are returned as the first and second elements
    of each tuple, whereas other section dividers (typically indicating the end
    of a section) employ None as the first and second elements. The span (start
    offset and end offset of the divider) is provided as a tuple in the third
    element of each result tuple.
    """

    headings = []

    for match in section_regexp.finditer(body):
        level = match.group("level")
        level = level and len(match.group("level"))

        if (min_level is None or min_level <= level) and \
            (max_level is None or level <= max_level) or \
            level is None:

            headings.append((match.group("heading"), level, match.span()))

    return headings

def getCategoryMembership(body):

    "From the given 'body', return the categories the page belongs to."

    match = category_membership_regexp.search(body)
    if match:
        return [x for x in match.groups() if x]
    else:
        return []

def getCategoryDeclarations(body):

    """
    From the given 'body', return the category declaration sections in the page
    in the form of a list of tuples, each containing a list of categories, the
    start of the declaration region and the end of the region.
    """

    return [([x for x in match.groups() if x], match.span()) for match in category_declarations_regexp.finditer(body)]

def makeCategoryDeclaration(categories):

    "Return a category declaration string for the given 'categories'."

    return "\n----\n%s\n" % " ".join(categories)

# vim: tabstop=4 expandtab shiftwidth=4
